@@ -1,16 +1,36 @@
 import itertools
+import inspect
 from functools import lru_cache
-from typing import Union
+from typing import Union, Callable, Tuple
+import warnings
 
 import numpy as np
 import pandas as pd
-from joblib import Parallel
+import joblib
 from scipy.special import comb
 from sklearn.preprocessing import PowerTransformer, StandardScaler
 from tqdm.auto import tqdm
+from rnalysis.utils import parsing
+
+try:
+    import numba
+except ImportError:
+    warnings.warn("RNAlysis can perform faster when package 'numba' is installed. \n"
+                  "If you want to improve the performance of slow operations on RNAlysis, "
+                  "please install package 'numba'. ")
 
 
-class ProgressParallel(Parallel):
+    class numba:
+        @staticmethod
+        def jit(*args, **kwargs):
+            return lambda f: f
+
+        @staticmethod
+        def njit(*args, **kwargs):
+            return lambda f: f
+
+
+class ProgressParallel(joblib.Parallel):
     # tqdm progress bar for parallel tasks based on:
     # https://stackoverflow.com/questions/37804279/how-can-we-use-tqdm-in-a-parallel-execution-with-joblib/50925708
     # answer by 'user394430'
@@ -19,13 +39,14 @@ class ProgressParallel(Parallel):
         self._total = total
         self._desc = desc
         self._unit = unit
-        super().__init__(*args, **kwargs)
+        kwargs['n_jobs'] = -2
+        super().__init__(*args, **kwargs, verbose=100)
 
     def __call__(self, *args, **kwargs):
         fmt = '{desc}: {percentage:3.0f}%|{bar}| [{elapsed}<{remaining}, {rate_fmt}{postfix}]'
         with tqdm(disable=not self._use_tqdm, total=self._total, desc=self._desc, unit=self._unit,
                   bar_format=fmt) as self._pbar:
-            return Parallel.__call__(self, *args, **kwargs)
+            return joblib.Parallel.__call__(self, *args, **kwargs)
 
     def print_progress(self):
         if self._total is None:
@@ -114,3 +135,45 @@ class SetWithMajorityVote(set):
                 counts[obj] = counts.get(obj, 0) + (1 / n_sets)
         result = {key for key, val in counts.items() if val >= majority_threshold}
         return result
+
+
+def get_method_signature(method: Union[str, Callable], obj: object = None):
+    try:
+        if isinstance(method, str):
+            func = getattr(obj, method)
+        else:
+            func = method
+        signature = inspect.signature(func)
+        return signature.parameters
+    except AttributeError:
+        return {}
+
+
+def get_method_docstring(method: Union[str, Callable], obj: object = None) -> Tuple[str, dict]:
+    try:
+        if isinstance(method, str):
+            func = getattr(obj, method)
+        else:
+            func = method
+        raw_docstring = inspect.cleandoc(inspect.getdoc(func))
+        return parsing.parse_docstring(raw_docstring)
+    except AttributeError:
+        return '', {}
+
+
+def despine(ax):
+    for side in ['top', 'right']:
+        ax.spines[side].set_visible(False)
+
+
+def mix_colors(*colors: Tuple[float, float, float]):
+    n_colors = len(colors)
+    if n_colors == 1:
+        return colors[0]
+
+    colors = np.array(colors)
+    multiplier = (1 / n_colors) + (1.6 / (2 ** n_colors)) / n_colors
+
+    mix_color = multiplier * np.sum(colors, axis=0)
+    mix_color = np.min([mix_color, [1.0, 1.0, 1.0]], 0)
+    return mix_color
